// RUN: iree-opt -allow-unregistered-dialect -split-input-file -canonicalize %s | iree-opt -allow-unregistered-dialect -split-input-file | IreeFileCheck %s

// CHECK-LABEL: @workgroupStaticShapeDims
func @workgroupStaticShapeDims(%arg0 : tensor<?x4xf32>) -> tensor<4x?xf32> {
  %c128 = arith.constant 128 : index
  %x = arith.constant 100 : index
  %y = arith.constant 50 : index
  // CHECK: flow.dispatch.workgroups
  %0 = flow.dispatch.workgroups[%x, %y](%arg0) : (tensor<?x4xf32>{%c128}) -> (tensor<4x?xf32>{%c128}) = (
    // CHECK-NEXT: (%[[ARG0:.+]]: !flow.dispatch.tensor<readonly:?x4xf32>,
    %arg0_capture: !flow.dispatch.tensor<readonly:?x4xf32>,
    // CHECK-SAME:  %[[RET0:.+]]: !flow.dispatch.tensor<writeonly:4x?xf32>)
    %ret0: !flow.dispatch.tensor<writeonly:4x?xf32>
  ) {
    // CHECK: %[[DIM_4:.+]] = arith.constant 4 : index

    // CHECK: %[[ARG0_SHAPE:.+]] = flow.dispatch.shape %[[ARG0]]
    %arg0_shape = flow.dispatch.shape %arg0_capture : !flow.dispatch.tensor<readonly:?x4xf32> -> !shapex.ranked_shape<[?,4]>
    // CHECK: %[[ARG0_DIM0:.+]] = shapex.ranked_dim %[[ARG0_SHAPE]][0]
    %arg0_dim0 = shapex.ranked_dim %arg0_shape[0] : !shapex.ranked_shape<[?,4]> -> index
    %arg0_dim1 = shapex.ranked_dim %arg0_shape[1] : !shapex.ranked_shape<[?,4]> -> index
    // CHECK: "test.sink"(%[[ARG0_DIM0]], %[[DIM_4]])
    "test.sink"(%arg0_dim0, %arg0_dim1) : (index, index) -> ()

    // CHECK: %[[RET0_SHAPE:.+]] = flow.dispatch.shape %[[RET0]]
    %ret0_shape = flow.dispatch.shape %ret0 : !flow.dispatch.tensor<writeonly:4x?xf32> -> !shapex.ranked_shape<[4,?]>
    %ret0_dim0 = shapex.ranked_dim %ret0_shape[0] : !shapex.ranked_shape<[4,?]> -> index
    // CHECK: %[[RET0_DIM1:.+]] = shapex.ranked_dim %[[RET0_SHAPE]][1]
    %ret0_dim1 = shapex.ranked_dim %ret0_shape[1] : !shapex.ranked_shape<[4,?]> -> index
    // CHECK: "test.sink"(%[[DIM_4]], %[[RET0_DIM1]])
    "test.sink"(%ret0_dim0, %ret0_dim1) : (index, index) -> ()

    flow.return
  }
  return %0 : tensor<4x?xf32>
}

// -----

// CHECK-LABEL: @workgroupRankFolding
func @workgroupRankFolding(%arg0 : tensor<?x4xf32>) -> tensor<4x?xf32> {
  %c128 = arith.constant 128 : index
  %x = arith.constant 100 : index
  %y = arith.constant 50 : index
  // CHECK: flow.dispatch.workgroups
  %0 = flow.dispatch.workgroups[%x, %y](%arg0) : (tensor<?x4xf32>{%c128}) -> (tensor<4x?xf32>{%c128}) = (
    %arg0_capture: !flow.dispatch.tensor<readonly:?x4xf32>,
    %ret0: !flow.dispatch.tensor<writeonly:4x?xf32>
  ) {
    // CHECK: %[[RANK:.+]] = arith.constant 2 : index
    %workgroup_rank = flow.dispatch.workgroup.rank : index
    // CHECK-NEXT: "test.sink"(%[[RANK]])
    "test.sink"(%workgroup_rank) : (index) -> ()
    flow.return
  }
  return %0 : tensor<4x?xf32>
}

// -----

// CHECK-LABEL: @convertDimOfDispatchInputLoadToDispatchShape
// CHECK-SAME:    %[[ARG:.*]]: !flow.dispatch.tensor<readonly:?xf32>) {
func @convertDimOfDispatchInputLoadToDispatchShape(%arg0: !flow.dispatch.tensor<readonly:?xf32>) {
  // CHECK-NEXT: %[[RANKED_SHAPE:.*]] = flow.dispatch.shape %[[ARG]]
  // CHECK-NEXT: %[[DIM:.*]] = shapex.ranked_dim %[[RANKED_SHAPE]][0]
  // CHECK-NEXT: "test.sink"(%[[DIM]]) : (index) -> ()
  %tensor = flow.dispatch.tensor.load %arg0, offsets=[], sizes=[], strides=[] : !flow.dispatch.tensor<readonly:?xf32> -> tensor<?xf32>
  %c0 = arith.constant 0 : index
  %dim = tensor.dim %tensor, %c0 : tensor<?xf32>
  "test.sink"(%dim) : (index) -> ()
  return
}

// -----

// CHECK-LABEL: @inlineWithTiedResults1
// CHECK-SAME: (%[[ARG0:.+]]: tensor<1x4xf32>)
func @inlineWithTiedResults1(%arg0: tensor<1x4xf32>) -> tensor<1x4xf32> {
  // CHECK-NOT: constant 128
  %cst = arith.constant 128 : index
  // CHECK-DAG: %[[X:.+]] = arith.constant 100
  %x = arith.constant 100 : index
  // CHECK-DAG: %[[Y:.+]] = arith.constant 50
  %y = arith.constant 50 : index
  //      CHECK: flow.dispatch.workgroups[%[[X]], %[[Y]]](%[[ARG0]]) : (tensor<1x4xf32>) -> %[[ARG0]] =
  // CHECK-NEXT:   (%[[ARG0_INNER:.+]]: !flow.dispatch.tensor<readwrite:1x4xf32>)
  %0 = flow.dispatch.workgroups[%x, %y](%cst, %arg0) : (index, tensor<1x4xf32>) -> %arg0 = (
    %cst_capture: index,
    %arg0_capture: !flow.dispatch.tensor<readwrite:1x4xf32>
  ) {
    //      CHECK: %[[INLINED_CST:.+]] = arith.constant 128 : index
    // CHECK-NEXT: "test.sink"(%[[INLINED_CST]])
    "test.sink"(%cst_capture) : (index) -> ()
    // CHECK-NEXT: "test.sink"(%[[ARG0_INNER]])
    "test.sink"(%arg0_capture) : (!flow.dispatch.tensor<readwrite:1x4xf32>) -> ()
    flow.return
  }
  return %0 : tensor<1x4xf32>
}

// -----

// CHECK-LABEL: @inlineWithTiedResults2
// CHECK-SAME: (%[[ARG0:.+]]: tensor<1x4xf32>)
func @inlineWithTiedResults2(%arg0: tensor<1x4xf32>) -> tensor<1x4xf32> {
  // CHECK-NOT: constant 128
  %cst = arith.constant 128 : index
  // CHECK-DAG: %[[X:.+]] = arith.constant 100
  %x = arith.constant 100 : index
  // CHECK-DAG: %[[Y:.+]] = arith.constant 50
  %y = arith.constant 50 : index
  //      CHECK: flow.dispatch.workgroups[%[[X]], %[[Y]]](%[[ARG0]]) : (tensor<1x4xf32>) -> %[[ARG0]] =
  // CHECK-NEXT:   (%[[ARG0_INNER:.+]]: !flow.dispatch.tensor<readwrite:1x4xf32>)
  %0 = flow.dispatch.workgroups[%x, %y](%arg0, %cst) : (tensor<1x4xf32>, index) -> %arg0 = (
    %arg0_capture: !flow.dispatch.tensor<readwrite:1x4xf32>,
    %cst_capture: index
  ) {
    //      CHECK: %[[INLINED_CST:.+]] = arith.constant 128 : index
    // CHECK-NEXT: "test.sink"(%[[INLINED_CST]])
    "test.sink"(%cst_capture) : (index) -> ()
    // CHECK-NEXT: "test.sink"(%[[ARG0_INNER]])
    "test.sink"(%arg0_capture) : (!flow.dispatch.tensor<readwrite:1x4xf32>) -> ()
    flow.return
  }
  return %0 : tensor<1x4xf32>
}

// -----

// CHECK-LABEL: func @dontInlineReadWrite
// CHECK-SAME: (%[[ARG0:.+]]: tensor<1x4xf32>)
func @dontInlineReadWrite(%arg0: tensor<1x4xf32>) -> tensor<4x8xf32> {
  // CHECK: %[[CST:.+]] = arith.constant dense<0.000000e+00> : tensor<4x8xf32>
  %cst = arith.constant dense<0.0> : tensor<4x8xf32>
  %x = arith.constant 100 : index
  %y = arith.constant 50 : index
  //      CHECK: flow.dispatch.workgroups[{{.+}}](%[[ARG0]], %[[CST]]) : (tensor<1x4xf32>, tensor<4x8xf32>) -> %cst
  // CHECK-NEXT:   (%{{.+}}: !flow.dispatch.tensor<readonly:1x4xf32>, %{{.+}}: !flow.dispatch.tensor<readwrite:4x8xf32>)
  %0 = flow.dispatch.workgroups[%x, %y](%arg0, %cst) : (tensor<1x4xf32>, tensor<4x8xf32>) -> %cst = (
    %arg0_capture: !flow.dispatch.tensor<readonly:1x4xf32>,
    %arg1_capture: !flow.dispatch.tensor<readwrite:4x8xf32>
  ) {
    "test.sink"(%arg0_capture) : (!flow.dispatch.tensor<readonly:1x4xf32>) -> ()
    %load = flow.dispatch.tensor.load %arg1_capture, offsets=[], sizes=[], strides=[] : !flow.dispatch.tensor<readwrite:4x8xf32> -> tensor<4x8xf32>
    %0 = "test.do_work"(%load) : (tensor<4x8xf32>) -> (tensor<4x8xf32>)
    flow.dispatch.tensor.store %0, %arg1_capture, offsets=[], sizes=[], strides=[] : tensor<4x8xf32> -> !flow.dispatch.tensor<readwrite:4x8xf32>
    flow.return
  }
  return %0 : tensor<4x8xf32>
}

// -----

// CHECK-LABEL: func @remove_unused_result
func @remove_unused_result(%arg0 : tensor<9xi32>, %arg1 : tensor<9xi32>) -> (tensor<i32>) {
  %c1 = arith.constant 1 : index
  //      CHECK: flow.dispatch.workgroups[%c1, %c1, %c1]() : () -> tensor<i32> =
  // CHECK-NEXT:   (%{{.+}}: !flow.dispatch.tensor<writeonly:i32>)
  //      CHECK: flow.dispatch.tensor.store
  //  CHECK-NOT: flow.dispatch.tensor.store
  %0:2 = flow.dispatch.workgroups[%c1, %c1, %c1](%arg0, %arg1) : (tensor<9xi32>, tensor<9xi32>) -> (tensor<i32>, tensor<i32>) =
      (%arg0: !flow.dispatch.tensor<readonly:9xi32>, %arg1: !flow.dispatch.tensor<readonly:9xi32>, %arg2: !flow.dispatch.tensor<writeonly:i32>, %arg3: !flow.dispatch.tensor<writeonly:i32>) {
    %c0_i32 = arith.constant 0 : i32
    %c-2147483648_i32 = arith.constant -2147483648 : i32
    %0 = flow.dispatch.tensor.load %arg0, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:9xi32> -> tensor<9xi32>
    %1 = flow.dispatch.tensor.load %arg1, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:9xi32> -> tensor<9xi32>
    %2 = linalg.init_tensor [] : tensor<i32>
    %3 = linalg.fill(%c-2147483648_i32, %2) : i32, tensor<i32> -> tensor<i32>
    %4 = linalg.fill(%c0_i32, %2) : i32, tensor<i32> -> tensor<i32>
    flow.dispatch.tensor.store %3, %arg2, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<writeonly:i32>
    flow.dispatch.tensor.store %4, %arg3, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<writeonly:i32>
    flow.return
  }
  return %0#0 : tensor<i32>
}

// -----

// CHECK-LABEL: func @remove_unused_read_write_result
func @remove_unused_read_write_result(%arg0 : tensor<9xi32>, %arg1 : tensor<9xi32>) -> (tensor<i32>) {
  %c1 = arith.constant 1 : index
  //      CHECK: flow.dispatch.workgroups[%c1, %c1, %c1]() : () -> tensor<i32> =
  // CHECK-NEXT:   (%{{.+}}: !flow.dispatch.tensor<writeonly:i32>)
  //      CHECK: flow.dispatch.tensor.store %{{.+}},
  //  CHECK-NOT: flow.dispatch.tensor.store
  %0:2 = flow.dispatch.workgroups[%c1, %c1, %c1](%arg0, %arg1) : (tensor<9xi32>, tensor<9xi32>) -> (tensor<i32>, tensor<i32>) =
      (%arg0: !flow.dispatch.tensor<readonly:9xi32>, %arg1: !flow.dispatch.tensor<readonly:9xi32>, %arg2: !flow.dispatch.tensor<writeonly:i32>, %arg3: !flow.dispatch.tensor<readwrite:i32>) {
    %c0_i32 = arith.constant 0 : i32
    %c-2147483648_i32 = arith.constant -2147483648 : i32
    %0 = flow.dispatch.tensor.load %arg0, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:9xi32> -> tensor<9xi32>
    %1 = flow.dispatch.tensor.load %arg1, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:9xi32> -> tensor<9xi32>
    %2 = linalg.init_tensor [] : tensor<i32>
    %3 = linalg.fill(%c-2147483648_i32, %2) : i32, tensor<i32> -> tensor<i32>
    %4 = linalg.fill(%c0_i32, %2) : i32, tensor<i32> -> tensor<i32>
    flow.dispatch.tensor.store %3, %arg2, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<writeonly:i32>
    flow.dispatch.tensor.store %4, %arg3, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<readwrite:i32>
    flow.return
  }
  return %0#0 : tensor<i32>
}

// -----

// CHECK-LABEL: func @keep_used_read_write_result
func @keep_used_read_write_result(%arg0 : tensor<9xi32>, %arg1 : tensor<9xi32>) -> (tensor<i32>) {
  %c1 = arith.constant 1 : index
  //      CHECK: flow.dispatch.workgroups[%c1, %c1, %c1]() : () -> (tensor<i32>, tensor<i32>) =
  // CHECK-NEXT:   (%{{.+}}: !flow.dispatch.tensor<writeonly:i32>, %{{.+}}: !flow.dispatch.tensor<readwrite:i32>)
  %0:2 = flow.dispatch.workgroups[%c1, %c1, %c1](%arg0, %arg1) : (tensor<9xi32>, tensor<9xi32>) -> (tensor<i32>, tensor<i32>) =
      (%arg0: !flow.dispatch.tensor<readonly:9xi32>, %arg1: !flow.dispatch.tensor<readonly:9xi32>, %arg2: !flow.dispatch.tensor<writeonly:i32>, %arg3: !flow.dispatch.tensor<readwrite:i32>) {
    %c-2147483648_i32 = arith.constant -2147483648 : i32
    %0 = flow.dispatch.tensor.load %arg3, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readwrite:i32> -> tensor<i32>
    %val = tensor.extract %0[] : tensor<i32>
    %1 = flow.dispatch.tensor.load %arg1, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:9xi32> -> tensor<9xi32>
    %2 = linalg.init_tensor [] : tensor<i32>
    %3 = linalg.fill(%c-2147483648_i32, %2) : i32, tensor<i32> -> tensor<i32>
    %4 = linalg.fill(%val, %2) : i32, tensor<i32> -> tensor<i32>
    flow.dispatch.tensor.store %3, %arg2, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<writeonly:i32>
    flow.dispatch.tensor.store %4, %arg3, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<readwrite:i32>
    flow.return
  }
  return %0#0 : tensor<i32>
}

// -----

// CHECK-LABEL: func @inline_cst_and_remove_unused_read_write_result
func @inline_cst_and_remove_unused_read_write_result() -> tensor<i32> {
  %cst = arith.constant dense<0> : tensor<i32>
  %c1 = arith.constant 1 : index
  //      CHECK: flow.dispatch.workgroups[%c1, %c1, %c1]() : () -> tensor<i32> =
  // CHECK-NEXT:   (%{{.+}}: !flow.dispatch.tensor<readwrite:i32>)
  %0:2 = flow.dispatch.workgroups[%c1, %c1, %c1](%cst) : (tensor<i32>) -> (tensor<i32>, tensor<i32>) =
      (%arg0: !flow.dispatch.tensor<readonly:i32>, %arg1: !flow.dispatch.tensor<writeonly:i32>, %arg2: !flow.dispatch.tensor<readwrite:i32>) {
    %1 = flow.dispatch.tensor.load %arg0, offsets = [], sizes = [], strides = [] : !flow.dispatch.tensor<readonly:i32> -> tensor<i32>
    "test.sink"(%1) : (tensor<i32>) -> ()
    %c0_i32 = arith.constant 0 : i32
    %c-2147483648_i32 = arith.constant -2147483648 : i32
    %2 = linalg.init_tensor [] : tensor<i32>
    %3 = linalg.fill(%c-2147483648_i32, %2) : i32, tensor<i32> -> tensor<i32>
    %4 = linalg.fill(%c0_i32, %2) : i32, tensor<i32> -> tensor<i32>
    flow.dispatch.tensor.store %3, %arg1, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<writeonly:i32>
    flow.dispatch.tensor.store %4, %arg2, offsets = [], sizes = [], strides = [] : tensor<i32> -> !flow.dispatch.tensor<readwrite:i32>
    flow.return
  }
  return %0#1 : tensor<i32>
}
